@@ -1,5 +1,5 @@
 /* giffunc.c - General functions for the GIF library.
-   Copyright (C) 1997-2011 Eddie Kohler, kohler@cs.ucla.edu
+   Copyright (C) 1997-2013 Eddie Kohler, kohler@cs.ucla.edu
    This file is part of the LCDF GIF library.
 
    The LCDF GIF library is free software. It is distributed under the GNU
@@ -325,7 +325,6 @@ Gif_CopyStreamImages(Gif_Stream *gfs)
 Gif_Colormap *
 Gif_CopyColormap(Gif_Colormap *src)
 {
-  int i;
   Gif_Colormap *dest;
   if (!src)
     return 0;
@@ -334,11 +333,7 @@ Gif_CopyColormap(Gif_Colormap *src)
   if (!dest)
     return 0;
 
-  for (i = 0; i < src->ncol; i++) {
-    dest->col[i] = src->col[i];
-    dest->col[i].haspixel = 0;
-  }
-
+  memcpy(dest->col, src->col, sizeof(src->col[0]) * src->ncol);
   return dest;
 }
 
@@ -770,6 +765,12 @@ Gif_CreateUncompressedImage(Gif_Image *gfi)
 				  gfi->interlace);
 }
 
+void
+Gif_InitCompressInfo(Gif_CompressInfo *gcinfo)
+{
+    gcinfo->flags = 0;
+}
+
 
 void
 Gif_Debug(char *x, ...)
