@@ -109,10 +109,12 @@ static int any_output_successful = 0;
 #define CH_COLOR_TRANSFORM	9
 #define CH_RESIZE		10
 #define CH_MEMORY		11
+#define CH_GAMMA		12
 static const char *output_option_types[] = {
   "loopcount", "logical screen", "optimization", "output file",
   "colormap size", "dither", "colormap", "colormap method",
-  "background", "color transformation", "resize", "memory conservation"
+  "background", "color transformation", "resize", "memory conservation",
+  "gamma"
 };
 
 
@@ -182,6 +184,8 @@ static const char *output_option_types[] = {
 #define RESIZE_FIT_WIDTH_OPT	365
 #define RESIZE_FIT_HEIGHT_OPT	366
 #define SIZE_INFO_OPT		367
+#define GAMMA_OPT		368
+#define GRAY_OPT		369
 
 #define LOOP_TYPE		(Clp_ValFirstUser)
 #define DISPOSAL_TYPE		(Clp_ValFirstUser + 1)
@@ -220,7 +224,8 @@ const Clp_Option options[] = {
   { "delay", 'd', 'd', Clp_ValInt, Clp_Negate },
   { "delete", 0, DELETE_OPT, 0, 0 },
   { "disposal", 'D', DISPOSAL_OPT, DISPOSAL_TYPE, Clp_Negate },
-  { "dither", 'f', DITHER_OPT, 0, Clp_Negate },
+  { 0, 'f', DITHER_OPT, 0, Clp_Negate },
+  { "dither", 0, DITHER_OPT, Clp_ValString, Clp_Negate | Clp_Optional },
   { "done", 0, ALTER_DONE_OPT, 0, 0 },
 
   { "explode", 'e', 'e', 0, 0 },
@@ -233,6 +238,9 @@ const Clp_Option options[] = {
   { "flip-vertical", 0, FLIP_VERT_OPT, 0, Clp_Negate },
   { "no-flip", 0, NO_FLIP_OPT, 0, 0 },
 
+  { "gamma", 0, GAMMA_OPT, Clp_ValString, Clp_Negate },
+  { "gray", 0, GRAY_OPT, 0, 0 },
+
   { "help", 'h', HELP_OPT, 0, 0 },
 
   { "info", 'I', INFO_OPT, 0, Clp_Negate },
@@ -780,27 +788,21 @@ set_new_fixed_colormap(const char *name)
     def_output_data.colormap_fixed = read_colormap_file(name, 0);
 }
 
-static void
-do_set_colormap(Gif_Stream *gfs, Gif_Colormap *gfcm)
-{
-  colormap_image_func image_func;
-  if (active_output_data.colormap_dither)
-    image_func = colormap_image_floyd_steinberg;
-  else
-    image_func = colormap_image_posterize;
-  colormap_stream(gfs, gfcm, image_func);
-}
-
 static void
 do_colormap_change(Gif_Stream *gfs)
 {
+  if (active_output_data.colormap_fixed || active_output_data.colormap_size > 0)
+    kc_set_gamma(active_output_data.colormap_gamma_type,
+                 active_output_data.colormap_gamma);
+
   if (active_output_data.colormap_fixed)
-    do_set_colormap(gfs, active_output_data.colormap_fixed);
+    colormap_stream(gfs, active_output_data.colormap_fixed,
+                    &active_output_data);
 
   if (active_output_data.colormap_size > 0) {
     int nhist;
     Gif_Color *hist;
-    Gif_Colormap *(*adapt_func)(Gif_Color *, int, int, int);
+    Gif_Colormap* (*adapt_func)(Gif_Color*, int, Gt_OutputData*);
     Gif_Colormap *new_cm;
 
     /* set up the histogram */
@@ -837,9 +839,8 @@ do_colormap_change(Gif_Stream *gfs)
 
     }
 
-    new_cm = (*adapt_func)(hist, nhist, active_output_data.colormap_size,
-                           !active_output_data.colormap_fixed);
-    do_set_colormap(gfs, new_cm);
+    new_cm = (*adapt_func)(hist, nhist, &active_output_data);
+    colormap_stream(gfs, new_cm, &active_output_data);
 
     Gif_DeleteArray(hist);
     Gif_DeleteColormap(new_cm);
@@ -1148,7 +1149,10 @@ initialize_def_frame(void)
   def_output_data.colormap_size = 0;
   def_output_data.colormap_fixed = 0;
   def_output_data.colormap_algorithm = COLORMAP_DIVERSITY;
-  def_output_data.colormap_dither = 0;
+  def_output_data.dither_type = dither_none;
+  def_output_data.dither_name = "none";
+  def_output_data.colormap_gamma_type = KC_GAMMA_SRGB;
+  def_output_data.colormap_gamma = 2.2;
 
   def_output_data.optimizing = 0;
   def_output_data.scaling = GT_SCALING_NONE;
@@ -1189,7 +1193,16 @@ combine_output_options(void)
     Gif_DeleteColormap(active_output_data.colormap_fixed);
     active_output_data.colormap_fixed = def_output_data.colormap_fixed;
   }
-  COMBINE_ONE_OUTPUT_OPTION(CH_DITHER, colormap_dither);
+  if (CHANGED(recent, CH_DITHER)) {
+    MARK_CH(output, CH_DITHER);
+    active_output_data.dither_type = def_output_data.dither_type;
+    active_output_data.dither_data = def_output_data.dither_data;
+  }
+  if (CHANGED(recent, CH_GAMMA)) {
+    MARK_CH(output, CH_GAMMA);
+    active_output_data.colormap_gamma_type = def_output_data.colormap_gamma_type;
+    active_output_data.colormap_gamma = def_output_data.colormap_gamma;
+  }
 
   if (CHANGED(recent, CH_RESIZE)) {
     MARK_CH(output, CH_RESIZE);
@@ -1258,6 +1271,8 @@ copy_crop(Gt_Crop *oc)
 int
 main(int argc, char *argv[])
 {
+  Clp_Parser* clp;
+
   /* Check SIZEOF constants (useful for Windows). If these assertions fail,
      you've used the wrong Makefile. You should've used Makefile.w32 for
      32-bit Windows and Makefile.w64 for 64-bit Windows. */
@@ -1265,8 +1280,7 @@ main(int argc, char *argv[])
   static_assert(sizeof(unsigned long) == SIZEOF_UNSIGNED_LONG, "unsigned long has the wrong size.");
   static_assert(sizeof(void*) == SIZEOF_VOID_P, "void* has the wrong size.");
 
-  Clp_Parser *clp =
-    Clp_NewParser(argc, (const char * const *)argv, sizeof(options) / sizeof(options[0]), options);
+  clp = Clp_NewParser(argc, (const char * const *)argv, sizeof(options) / sizeof(options[0]), options);
 
   Clp_AddStringListType
     (clp, LOOP_TYPE, Clp_AllowNumbers,
@@ -1704,7 +1718,13 @@ main(int argc, char *argv[])
       }
       break;
 
-     case USE_COLORMAP_OPT:
+     case GRAY_OPT:
+      MARK_CH(output, CH_USE_COLORMAP);
+      Gif_DeleteColormap(def_output_data.colormap_fixed);
+      set_new_fixed_colormap("gray");
+      break;
+
+    case USE_COLORMAP_OPT:
       MARK_CH(output, CH_USE_COLORMAP);
       Gif_DeleteColormap(def_output_data.colormap_fixed);
       if (clp->negated)
@@ -1718,10 +1738,50 @@ main(int argc, char *argv[])
       def_output_data.colormap_algorithm = clp->val.i;
       break;
 
-     case DITHER_OPT:
-      MARK_CH(output, CH_DITHER);
-      def_output_data.colormap_dither = !clp->negated;
+    case DITHER_OPT: {
+      const char* name;
+      if (clp->negated)
+        name = "none";
+      else if (!clp->have_val)
+        name = "default";
+      else
+        name = clp->val.s;
+      if (strcmp(name, "posterize") == 0)
+        name = "none";
+      if (strcmp(name, def_output_data.dither_name) != 0
+          && (strcmp(name, "none") == 0
+              || strcmp(def_output_data.dither_name, "default") != 0))
+        MARK_CH(output, CH_DITHER);
+      UNCHECKED_MARK_CH(output, CH_DITHER);
+      if (set_dither_type(&def_output_data, name) < 0)
+        Clp_OptionError(clp, "%<%s%> is not a valid dither", name);
+      def_output_data.dither_name = name;
       break;
+    }
+
+    case GAMMA_OPT: {
+#if HAVE_POW
+      char* ends;
+      MARK_CH(output, CH_GAMMA);
+      if (clp->negated) {
+        def_output_data.colormap_gamma_type = KC_GAMMA_NUMERIC;
+        def_output_data.colormap_gamma = 1;
+      } else if (strcmp(clp->val.s, "sRGB") == 0
+                 || strcmp(clp->val.s, "srgb") == 0)
+        def_output_data.colormap_gamma_type = KC_GAMMA_SRGB;
+      else {
+        double gamma = strtod(clp->val.s, &ends);
+        if (*clp->val.s && !*ends && !isspace((unsigned char) *clp->val.s)) {
+          def_output_data.colormap_gamma_type = KC_GAMMA_NUMERIC;
+          def_output_data.colormap_gamma = gamma;
+        } else
+          Clp_OptionError(clp, "%O should be a number or %<srgb%>");
+      }
+#else
+      Clp_OptionError(clp, "this version of Gifsicle does not support %O");
+#endif
+      break;
+    }
 
     case RESIZE_OPT:
     case RESIZE_FIT_OPT:
