@@ -1,5 +1,5 @@
 /* gifunopt.c - Unoptimization function for the GIF library.
-   Copyright (C) 1997-2011 Eddie Kohler, kohler@cs.ucla.edu
+   Copyright (C) 1997-2013 Eddie Kohler, kohler@cs.ucla.edu
    This file is part of the LCDF GIF library.
 
    The LCDF GIF library is free software. It is distributed under the GNU
@@ -26,8 +26,10 @@ put_image_in_screen(Gif_Stream *gfs, Gif_Image *gfi, uint16_t *screen)
   int x, y;
   int w = gfi->width;
   int h = gfi->height;
-  if (gfi->left + w > gfs->screen_width) w = gfs->screen_width - gfi->left;
-  if (gfi->top + h > gfs->screen_height) h = gfs->screen_height - gfi->top;
+  if (gfi->left + w > gfs->screen_width)
+      w = gfs->screen_width - gfi->left;
+  if (gfi->top + h > gfs->screen_height)
+      h = gfs->screen_height - gfi->top;
 
   for (y = 0; y < h; y++) {
     uint16_t *move = screen + gfs->screen_width * (y + gfi->top) + gfi->left;
