@@ -44,8 +44,6 @@
 #define EXTERN extern
 #endif
 
-#define SAVE_FRAMES 20
-
 /*****
  * TIME STUFF (from xwrits)
  **/
@@ -99,6 +97,9 @@ struct timeval genesis_time;
  * THE VIEWER STRUCTURE
  **/
 
+static unsigned pixel_memory_limit_kb = 40000;
+static unsigned pixel_memory_kb;
+
 typedef struct Gt_Viewer {
 
   Display *display;
@@ -130,9 +131,9 @@ typedef struct Gt_Viewer {
   Pixmap pixmap;
   int im_pos;
   int was_unoptimized;
-  int free_pixmaps;
 
   Gif_XFrame *unoptimized_frames;
+  int n_unoptimized_frames;
 
   struct Gt_Viewer *next;
 
@@ -167,6 +168,7 @@ static int unoptimizing = 0;
 static int install_colormap = 0;
 static int interactive = 1;
 static int min_delay = 0;
+static int fallback_delay = 0;
 
 static struct timeval preparation_time;
 
@@ -185,6 +187,8 @@ static struct timeval preparation_time;
 #define NEW_WINDOW_OPT		311
 #define TITLE_OPT		312
 #define MIN_DELAY_OPT		313
+#define FALLBACK_DELAY_OPT	314
+#define MEMORY_LIMIT_OPT	315
 
 #define WINDOW_TYPE		(Clp_ValFirstUser)
 
@@ -197,7 +201,9 @@ const Clp_Option options[] = {
   { "install-colormap", 'i', INSTALL_COLORMAP_OPT, 0, Clp_Negate },
   { "interactive", 'e', INTERACTIVE_OPT, 0, Clp_Negate },
   { "help", 0, HELP_OPT, 0, 0 },
+  { "memory-limit", 0, MEMORY_LIMIT_OPT, Clp_ValUnsigned, Clp_Negate },
   { "min-delay", 0, MIN_DELAY_OPT, Clp_ValInt, Clp_Negate },
+  { "fallback-delay", 0, FALLBACK_DELAY_OPT, Clp_ValInt, Clp_Negate },
   { "name", 0, NAME_OPT, Clp_ValString, 0 },
   { "title", 'T', TITLE_OPT, Clp_ValString, 0 },
   { "unoptimize", 'U', UNOPTIMIZE_OPT, 0, Clp_Negate },
@@ -257,26 +263,30 @@ usage(void)
 'Gifview' is a lightweight GIF viewer for X. It can display animated GIFs as\n\
 slideshows, one frame at a time, or as animations.\n\
 \n\
-Usage: %s [--display DISPLAY] [OPTION]... [FILE | FRAME]...\n\
-\n\
+Usage: %s [--display DISPLAY] [OPTION]... [FILE | FRAME]...\n\n", program_name);
+  printf("\
 Options are:\n\
   -a, --animate                 Animate multiframe GIFs.\n\
   -U, --unoptimize              Unoptimize displayed GIFs.\n\
   -d, --display DISPLAY         Set display to DISPLAY.\n\
       --name NAME               Set application resource name to NAME.\n\
   -g, --geometry GEOMETRY       Set window geometry.\n\
-  -T, --title TITLE             Set window title.\n\
+  -T, --title TITLE             Set window title.\n");
+  printf("\
   -w, --window WINDOW           Show GIF in existing WINDOW.\n\
       --new-window WINDOW       Show GIF in new child of existing WINDOW.\n\
   -i, --install-colormap        Use a private colormap.\n\
   --bg, --background COLOR      Use COLOR for transparent pixels.\n\
       --min-delay DELAY         Set minimum frame delay to DELAY/100 sec.\n\
-  +e, --no-interactive          Ignore buttons and keystrokes.\n\
+      --fallback-delay DELAY    Set fallback frame delay to DELAY/100 sec.\n\
+  +e, --no-interactive          Ignore buttons and keystrokes.\n");
+  printf("\
+      --memory-limit LIM        Cache at most LIM megabytes of animation.\n\
       --help                    Print this message and exit.\n\
       --version                 Print version number and exit.\n\
 \n\
-Frame selections:               #num, #num1-num2, #num1-, #name\n\
-\n\
+Frame selections:               #num, #num1-num2, #num1-, #name\n\n");
+  printf("\
 Keystrokes:\n\
   [N]/[Space] Go to next frame.         [P]/[B] Go to previous frame.\n\
   [R]/[<] Go to first frame.            [>] Go to last frame.\n\
@@ -286,7 +296,7 @@ Keystrokes:\n\
 \n\
 Left mouse button goes to next frame, right mouse button deletes window.\n\
 \n\
-Report bugs to <ekohler@gmail.com>.\n", program_name);
+Report bugs to <ekohler@gmail.com>.\n");
 }
 
 
@@ -472,8 +482,8 @@ new_viewer(Display *display, Gif_Stream *gfs, const char *name)
   viewer->pixmap = None;
   viewer->im_pos = -1;
   viewer->was_unoptimized = 0;
-  viewer->free_pixmaps = 1;
   viewer->unoptimized_frames = Gif_NewXFrames(gfs);
+  viewer->n_unoptimized_frames = 0;
   viewer->next = viewers;
   viewers = viewer;
   viewer->animating = 0;
@@ -631,6 +641,8 @@ schedule_next_frame(Gt_Viewer *viewer)
   struct timeval interval;
   int delay = viewer->im[viewer->im_pos]->delay;
   int next_pos = viewer->im_pos + 1;
+  if (delay < 1)
+    delay = fallback_delay;
   if (delay < min_delay)
     delay = min_delay;
   if (next_pos == viewer->nim)
@@ -852,6 +864,11 @@ set_viewer_name(Gt_Viewer *viewer, int slow_number)
   Gif_DeleteArray(strs[0]);
 }
 
+static unsigned screen_memory_kb(const Gt_Viewer* viewer) {
+  return 1 + ((unsigned) (viewer->gfs->screen_width
+                          * viewer->gfs->screen_height) / 334);
+}
+
 static Pixmap
 unoptimized_frame(Gt_Viewer *viewer, int frame, int slow)
 {
@@ -859,21 +876,34 @@ unoptimized_frame(Gt_Viewer *viewer, int frame, int slow)
   if (!viewer->unoptimized_frames[frame].pixmap) {
     (void) Gif_XNextImage(viewer->gfx, viewer->gfs, frame,
 			  viewer->unoptimized_frames);
+    pixel_memory_kb += screen_memory_kb(viewer);
+    viewer->unoptimized_frames[viewer->n_unoptimized_frames].user_data =
+      frame;
+    ++viewer->n_unoptimized_frames;
     if (slow) {
       set_viewer_name(viewer, frame);
       XFlush(viewer->display);
     }
   }
 
-  /* maybe kill some old frames */
-  if (viewer->free_pixmaps && frame % SAVE_FRAMES == 1) {
-    int kill, block_first = frame - 1, block_last = frame + SAVE_FRAMES - 1;
-    for (kill = 1; kill < viewer->nim; kill++)
-      if (viewer->unoptimized_frames[kill].pixmap && kill % 50 != 0
-	  && (kill < block_first || kill >= block_last)) {
-	XFreePixmap(viewer->display, viewer->unoptimized_frames[kill].pixmap);
-	viewer->unoptimized_frames[kill].pixmap = None;
-      }
+  /* kill some old frames if over the memory limit */
+  while (pixel_memory_limit_kb != (unsigned) -1
+         && pixel_memory_limit_kb < pixel_memory_kb
+         && viewer->n_unoptimized_frames > 1) {
+    int killidx, killframe, i = 0;
+    do {
+      killidx = random() % viewer->n_unoptimized_frames;
+      killframe = viewer->unoptimized_frames[killidx].user_data;
+      ++i;
+    } while (killframe == frame
+             || (i < 10 && killframe > frame && killframe < frame + 5)
+             || (i < 10 && (killframe % 50) == 0));
+    XFreePixmap(viewer->display, viewer->unoptimized_frames[killframe].pixmap);
+    viewer->unoptimized_frames[killframe].pixmap = None;
+    --viewer->n_unoptimized_frames;
+    viewer->unoptimized_frames[killidx].user_data =
+      viewer->unoptimized_frames[viewer->n_unoptimized_frames].user_data;
+    pixel_memory_kb -= screen_memory_kb(viewer);
   }
 
   return viewer->unoptimized_frames[frame].pixmap;
@@ -903,7 +933,7 @@ prepare_frame(Gt_Viewer *viewer, int frame)
   }
 
   /* Prepare the frame */
-  (void) unoptimized_frame(viewer, frame, changed_cursor);
+  (void) unoptimized_frame(viewer, frame, changed_cursor && !viewer->animating);
 
   /* Restore cursor */
   if (changed_cursor)
@@ -1313,6 +1343,17 @@ main(int argc, char *argv[])
       min_delay = clp->negated ? 0 : clp->val.i;
       break;
 
+    case FALLBACK_DELAY_OPT:
+      fallback_delay = clp->negated ? 0 : clp->val.i;
+      break;
+
+    case MEMORY_LIMIT_OPT:
+      if (clp->negated || clp->val.u >= ((unsigned) -1 / 1000))
+        pixel_memory_limit_kb = (unsigned) -1;
+      else
+        pixel_memory_limit_kb = clp->val.u * 1000;
+      break;
+
      case VERSION_OPT:
       printf("gifview (LCDF Gifsicle) %s\n", VERSION);
       printf("Copyright (C) 1997-2013 Eddie Kohler\n\
