@@ -607,9 +607,9 @@ write_color_table(Gif_Colormap *gfcm, int totalcol, Gif_Writer *grr)
   int i, ncol = gfcm->ncol;
 
   for (i = 0; i < ncol && i < totalcol; i++, c++) {
-    gifputbyte(c->red, grr);
-    gifputbyte(c->green, grr);
-    gifputbyte(c->blue, grr);
+    gifputbyte(c->gfc_red, grr);
+    gifputbyte(c->gfc_green, grr);
+    gifputbyte(c->gfc_blue, grr);
   }
 
   /* Pad out colormap with black. */
@@ -666,6 +666,11 @@ write_image(Gif_Stream *gfs, Gif_Image *gfi, Gif_CodeTable *gfc,
       compressed_len -= amt;
     }
 
+  } else if (!gfi->img) {
+    Gif_UncompressImage(gfi);
+    write_compressed_data(gfi, min_code_bits, gfc, grr);
+    Gif_ReleaseUncompressedImage(gfi);
+
   } else
     write_compressed_data(gfi, min_code_bits, gfc, grr);
 
